@@ -95,7 +95,7 @@ proc ::deken::versioncheck {version} {
 }
 
 ## put the current version of this package here:
-if { [::deken::versioncheck 0.5.1] } {
+if { [::deken::versioncheck 0.6.0] } {
 
 ## FIXXXXME only initialize vars if not yet set
 set ::deken::installpath {}
@@ -104,6 +104,8 @@ set ::deken::hideforeignarch 1
 set ::deken::show_readme 1
 set ::deken::remove_on_install 1
 set ::deken::add_to_path 0
+set ::deken::keep_package 0
+set ::deken::verify_sha256 1
 set ::deken::preferences::installpath {}
 set ::deken::preferences::userinstallpath {}
 set ::deken::preferences::platform {}
@@ -113,6 +115,8 @@ set ::deken::preferences::show_readme {}
 set ::deken::preferences::remove_on_install {}
 set ::deken::preferences::add_to_path {}
 set ::deken::preferences::add_to_path_temp {}
+set ::deken::preferences::keep_package {}
+set ::deken::preferences::verify_sha256 {}
 
 namespace eval ::deken:: {
     namespace export open_searchui
@@ -161,17 +165,26 @@ proc ::deken::utilities::is_writable_dir {path} {
     return false
 }
 
+proc ::deken::utilities::substpath {path} {
+    set result [string map "@PD_PATH@ $::sys_libdir" $path]
+    return $result
+}
+
+proc ::deken::utilities::verbose {level message} {
+    ::pdwindow::verbose ${level} "\[deken\]: "
+    ::pdwindow::verbose ${level} ${message}
+    ::pdwindow::verbose ${level} "\n"
+}
+
 if { [catch {package require zipfile::decode} ] } {
 proc ::deken::utilities::unzipper {zipfile {path .}} {
     ## this is w32 only
     if {$::tcl_platform(platform) eq "windows"} { } { return 0 }
-    if { "" eq $::deken::_vbsunzip } {
-        set ::deken::_vbsunzip [ file join [::deken::gettmpdir] deken_unzip.vbs ]
-    }
 
-    if {[file exists $::deken::_vbsunzip]} {} {
-        ## no script yet, create one
-        set script {
+    ## create script-file
+    set vbsscript [::deken::get_tmpfilename [::deken::gettmpdir] ".vbs" ]
+    set script {
+On Error Resume Next
 Set fso = CreateObject("Scripting.FileSystemObject")
 
 'The location of the zip file.
@@ -184,19 +197,25 @@ If NOT fso.FolderExists(ExtractTo) Then
    fso.CreateFolder(ExtractTo)
 End If
 
-'Extract the contants of the zip file.
+'Extract the contents of the zip file.
 set objShell = CreateObject("Shell.Application")
 set FilesInZip=objShell.NameSpace(ZipFile).items
 objShell.NameSpace(ExtractTo).CopyHere(FilesInZip)
+'In case of an error, exit
+If Err.Number <> 0 Then
+  Err.Clear
+  WScript.Quit 1
+End If
+
 Set fso = Nothing
 Set objShell = Nothing
 }
-        if {![catch {set fileId [open $::deken::_vbsunzip "w"]}]} {
+        if {![catch {set fileId [open $vbsscript "w"]}]} {
             puts $fileId $script
             close $fileId
         }
-    }
-    if {[file exists $::deken::_vbsunzip]} {} {
+
+    if {[file exists $vbsscript]} {} {
         ## still no script, give up
         return 0
     }
@@ -205,13 +224,16 @@ Set objShell = Nothing
     if { [ catch {
 	set zipfilezip ${zipfile}.zip
 	file rename ${zipfile} ${zipfilezip}
-	exec cscript $::deken::_vbsunzip "${zipfilezip}" .
+	exec cscript "${vbsscript}" "${zipfilezip}" .
 	file rename ${zipfilezip} ${zipfile}
     } stdout ]
      } {
-        ::pdwindow::debug "\[deken\] VBS-unzip: $::deken::_vbsunzip\n$stdout\n"
+        catch { file rename ${zipfilezip} ${zipfile} }
+        catch { file delete "${vbsscript}" }
+        ::pdwindow::debug "\[deken\] VBS-unzip($vbsscript): $stdout\n"
         return 0
     }
+    catch { file delete "${vbsscript}" }
     return 1
 }
 } { # successfully imported zipfile::decode
@@ -255,16 +277,25 @@ proc ::deken::utilities::extract {installdir filename fullpkgfile} {
         ::pdwindow::debug [_ "\[deken\] " ]
         ::pdwindow::debug [format [_ "Successfully unzipped %1\$s into %2\$s."] $filename $installdir ]
         ::pdwindow::debug "\n"
-        catch { file delete $fullpkgfile }
+        if { "$::deken::keep_package" } { } {
+            catch { file delete $fullpkgfile }
+        }
     } else {
         # Open both the fullpkgfile folder and the zipfile itself
         # NOTE: in tcl 8.6 it should be possible to use the zlib interface to actually do the unzip
         ::pdwindow::error [_ "\[deken\] Unable to extract package automatically." ]
+        ::pdwindow::error "\n"
         ::pdwindow::post [_ "Please perform the following steps manually:" ]
+        ::pdwindow::post "\n"
         ::pdwindow::post [format [_ "1. Unzip %s." ]  $fullpkgfile ]
+        ::pdwindow::post "\n  "
+        ::pdwindow::post [_ "You might need to change the file-extension from .dek to .zip" ]
+        ::pdwindow::post "\n"
         pd_menucommands::menu_openfile $fullpkgfile
         ::pdwindow::post [format [_ "2. Copy the contents into %s." ] $installdir]
-        ::pdwindow::post ""
+        ::pdwindow::post "\n"
+        ::pdwindow::post [format [_ "3. Remove %s. (optional)" ]  $fullpkgfile ]
+        ::pdwindow::post "\n"
         pd_menucommands::menu_openfile $installdir
     }
 }
@@ -285,6 +316,41 @@ proc ::deken::utilities::uninstall {path library} {
     }
     return 1
 }
+if { [catch {package require sha256} ] } {
+    proc ::deken::utilities::verify_sha256 {url pkgfile} {
+        ::deken::utilities::verbose 0 [format [_ "skipping SHA256 verification of %s." ] $url ]
+        return 1
+    }
+} {  # successfully imported sha256
+    proc ::deken::utilities::verify_sha256 {url pkgfile} {
+        set retval 1
+        if { [ catch {
+            set hash [string trim [string tolower [ ::sha2::sha256 -hex -filename $pkgfile ] ] ]
+            set hashfile [::deken::download_file ${url}.sha256 [::deken::get_tmpfilename [::deken::gettmpdir] ".sha256" ] ]
+            if { "$hashfile" eq "" } {
+                ::deken::utilities::verbose 0 [format [_ "unable to fetch reference SHA256 for %s." ] $url ]
+                set retval 1
+            } {
+                set fp [open $hashfile r]
+                set reference [string trim [string tolower [read $fp] ] ]
+                close $fp
+                catch { file delete $hashfile }
+                if { "${hash}" eq "${reference}" } {
+                    set retval 1
+                } {
+                    # SHA256 verification failed...
+                    set retval 0
+                }
+            }
+        } stdout ] } {
+            ::deken::utilities::verbose 0 "\[deken\] $stdout\n"
+            # unable to verify
+            ::deken::utilities::verbose 0 [format [_ "unable to perform SHA256 verification for %s." ] $url ]
+            set retval 1
+        }
+        return ${retval}
+    }
+}
 
 proc ::deken::utilities::rmrecursive {path} {
     # recursively remove ${path} if it exists, traversing into each directory
@@ -325,10 +391,12 @@ if { [ catch { set ::deken::installpath [::pd_guiprefs::read dekenpath] } stdout
         set ::deken::userplatform $platform
         set ::deken::hideforeignarch [::deken::utilities::bool $hide ]
     }
-    proc ::deken::set_install_options {remove readme add} {
+    proc ::deken::set_install_options {remove readme add keep verify256} {
         set ::deken::remove_on_install [::deken::utilities::bool $remove]
         set ::deken::show_readme [::deken::utilities::bool $readme]
         set ::deken::add_to_path [::deken::utilities::tristate $add 0 0]
+        set ::deken::keep_package [::deken::utilities::bool $keep]
+        set ::deken::verify_sha256 [::deken::utilities::bool $verify256]
     }
 } {
     # Pd has a generic preferences system, that we can use
@@ -347,15 +415,21 @@ if { [ catch { set ::deken::installpath [::pd_guiprefs::read dekenpath] } stdout
     }
     set ::deken::remove_on_install [::deken::utilities::bool [::pd_guiprefs::read deken_remove_on_install] 1]
     set ::deken::show_readme [::deken::utilities::bool [::pd_guiprefs::read deken_show_readme] 1]
+    set ::deken::keep_package [::deken::utilities::bool [::pd_guiprefs::read deken_keep_package] 0]
+    set ::deken::verify_sha256 [::deken::utilities::bool [::pd_guiprefs::read deken_verify_sha256] 1]
     set ::deken::add_to_path [::deken::utilities::tristate [::pd_guiprefs::read deken_add_to_path] ]
 
-    proc ::deken::set_install_options {remove readme path} {
+    proc ::deken::set_install_options {remove readme path keep verify256} {
         set ::deken::remove_on_install [::deken::utilities::bool $remove]
         set ::deken::show_readme [::deken::utilities::bool $readme]
         set ::deken::add_to_path [::deken::utilities::tristate $path]
+        set ::deken::keep_package [::deken::utilities::bool $keep]
+        set ::deken::verify_sha256 [::deken::utilities::bool $verify256]
         ::pd_guiprefs::write deken_remove_on_install "$::deken::remove_on_install"
         ::pd_guiprefs::write deken_show_readme "$::deken::show_readme"
         ::pd_guiprefs::write deken_add_to_path "$::deken::add_to_path"
+        ::pd_guiprefs::write deken_keep_package "$::deken::keep_package"
+        ::pd_guiprefs::write deken_verify_sha256 "$::deken::verify_sha256"
     }
 }
 
@@ -416,9 +490,9 @@ proc ::deken::gettmpdir {} {
     if {[_iswdir $tmpdir]} {return $tmpdir}
 }
 
-proc ::deken::get_tmpfilename {{path ""}} {
+proc ::deken::get_tmpfilename {{path ""} {ext ""}} {
     for {set i 0} {true} {incr i} {
-        set tmpfile [file join ${path} dekentmp.${i}]
+        set tmpfile [file join ${path} dekentmp.${i}${ext}]
         if {![file exists $tmpfile]} {
             return $tmpfile
         }
@@ -427,7 +501,8 @@ proc ::deken::get_tmpfilename {{path ""}} {
 
 proc ::deken::get_writable_dir {paths} {
     foreach p $paths {
-        if { [ ::deken::utilities::is_writable_dir $p ] } { return $p }
+        set xp [ ::deken::utilities::substpath $p ]
+        if { [ ::deken::utilities::is_writable_dir $xp ] } { return $p }
     }
     return
 }
@@ -501,7 +576,7 @@ if { "" != "$::current_plugin_loadpath" } {
 ::pdwindow::verbose 0 [format [_ "\[deken\] Platform detected: %s" ] [::deken::platform2string 1] ]
 ::pdwindow::verbose 0 "\n"
 
-# architectures that can be substituted for eachother
+# architectures that can be substituted for each other
 array set ::deken::architecture_substitutes {}
 set ::deken::architecture_substitutes(x86_64) [list "amd64" ]
 set ::deken::architecture_substitutes(amd64) [list "x86_64" ]
@@ -556,6 +631,11 @@ proc ::deken::versioncompare {a b} {
     return 0
 }
 
+proc ::deken::bind_globalshortcuts {toplevel} {
+    set closescript "destroy $toplevel"
+    bind $toplevel <$::modifier-Key-w> $closescript
+}
+
 proc ::deken::status {{msg ""}} {
     #variable mytoplevelref
     #$mytoplevelref.results insert end "$msg\n"
@@ -570,36 +650,48 @@ proc ::deken::status {{msg ""}} {
 }
 proc ::deken::scrollup {} {
     variable mytoplevelref
-    $mytoplevelref.results see 0.0
+    if { [winfo exists $mytoplevelref] } {
+        $mytoplevelref.results see 0.0
+    }
 }
 proc ::deken::post {msg {tag ""}} {
     variable mytoplevelref
-    $mytoplevelref.results insert end "$msg\n" $tag
-    $mytoplevelref.results see end
+    if { [winfo exists $mytoplevelref] } {
+        $mytoplevelref.results insert end "$msg\n" $tag
+        $mytoplevelref.results see end
+    }
 }
 proc ::deken::clearpost {} {
     variable mytoplevelref
-    $mytoplevelref.results delete 1.0 end
+    if { [winfo exists $mytoplevelref] } {
+        $mytoplevelref.results delete 1.0 end
+    }
 }
 proc ::deken::bind_posttag {tag key cmd} {
     variable mytoplevelref
-    $mytoplevelref.results tag bind $tag $key $cmd
+    if { [winfo exists $mytoplevelref] } {
+        $mytoplevelref.results tag bind $tag $key $cmd
+    }
 }
 proc ::deken::highlightable_posttag {tag} {
     variable mytoplevelref
-    ::deken::bind_posttag $tag <Enter> \
-        "$mytoplevelref.results tag add highlight [ $mytoplevelref.results tag ranges $tag ]"
-    ::deken::bind_posttag $tag <Leave> \
-        "$mytoplevelref.results tag remove highlight [ $mytoplevelref.results tag ranges $tag ]"
-    # make sure that the 'highlight' tag is topmost
-    $mytoplevelref.results tag raise highlight
+    if { [winfo exists $mytoplevelref] } {
+        ::deken::bind_posttag $tag <Enter> \
+            "$mytoplevelref.results tag add highlight [ $mytoplevelref.results tag ranges $tag ]"
+        ::deken::bind_posttag $tag <Leave> \
+            "$mytoplevelref.results tag remove highlight [ $mytoplevelref.results tag ranges $tag ]"
+        # make sure that the 'highlight' tag is topmost
+        $mytoplevelref.results tag raise highlight
+    }
 }
 proc ::deken::bind_postmenu {mytoplevel tag menus} {
     set cmd "::deken::result_contextmenu %W %x %y $menus"
-    if {$::windowingsystem eq "aqua"} {
-        $mytoplevel.results tag bind $tag <2> $cmd
-    } else {
-        $mytoplevel.results tag bind $tag <3> $cmd
+    if { [winfo exists $mytoplevel] } {
+        if {$::windowingsystem eq "aqua"} {
+            $mytoplevel.results tag bind $tag <2> $cmd
+        } else {
+            $mytoplevel.results tag bind $tag <3> $cmd
+        }
     }
 }
 proc ::deken::result_contextmenu {widget theX theY args} {
@@ -655,6 +747,7 @@ proc ::deken::open_searchui {mytoplevel} {
         raise $mytoplevel
     } else {
         ::deken::create_dialog $mytoplevel
+        ::deken::bind_globalshortcuts $mytoplevel
         $mytoplevel.results tag configure error -foreground red
         $mytoplevel.results tag configure warn -foreground orange
         $mytoplevel.results tag configure info -foreground grey
@@ -768,8 +861,9 @@ proc ::deken::preferences::path_doit {rdb ckb path {mkdir true}} {
 
 proc ::deken::preferences::create_pathentry {toplevel row var path {generic false}} {
     # only add absolute paths to the pathentries
+    set xpath [ ::deken::utilities::substpath $path ]
     if {! $generic} {
-        if { [file pathtype $path] != "absolute" } { return }
+        if { [file pathtype $xpath] != "absolute"} { return }
     }
 
     set rdb [::deken::utilities::newwidget ${toplevel}.path]
@@ -778,7 +872,7 @@ proc ::deken::preferences::create_pathentry {toplevel row var path {generic fals
 
     radiobutton ${rdb} -value ${path} -text "${path}" -variable $var
     frame ${pad}
-    button ${chk} -text "..." -command "::deken::preferences::path_doit ${rdb} ${chk} ${path}"
+    button ${chk} -text "..." -command "::deken::preferences::path_doit ${rdb} ${chk} ${xpath}"
 
     grid ${rdb} -sticky w    -row ${row} -column 2
     grid ${pad} -sticky ""   -row ${row} -column 1 -padx 10
@@ -786,7 +880,7 @@ proc ::deken::preferences::create_pathentry {toplevel row var path {generic fals
 
 
     if {! $generic} {
-        ::deken::preferences::path_doit ${rdb} ${chk} ${path} false
+        ::deken::preferences::path_doit ${rdb} ${chk} ${xpath} false
     }
     return [list ${rdb} ${chk}]
 }
@@ -797,6 +891,7 @@ proc ::deken::preferences::create {mytoplevel} {
     # in order to get the scrolling-region right!!!
     # this seems to be so wrong...
     bind $mytoplevel <Map> "::deken::preferences::mapped %W"
+    ::deken::bind_globalshortcuts $mytoplevel
 
     set ::deken::preferences::installpath $::deken::installpath
     set ::deken::preferences::hideforeignarch $::deken::hideforeignarch
@@ -812,6 +907,8 @@ proc ::deken::preferences::create {mytoplevel} {
     set ::deken::preferences::userinstallpath $::deken::installpath
 
     set ::deken::preferences::show_readme $::deken::show_readme
+    set ::deken::preferences::keep_package $::deken::keep_package
+    set ::deken::preferences::verify_sha256 $::deken::verify_sha256
     set ::deken::preferences::remove_on_install $::deken::remove_on_install
     set ::deken::preferences::add_to_path $::deken::add_to_path
     set ::deken::preferences::add_to_path_temp $::deken::preferences::add_to_path
@@ -869,7 +966,11 @@ proc ::deken::preferences::create {mytoplevel} {
         ::deken::preferences::create_pathpad ${pathsframe} ${row}
         incr row
     }
+    set extradir [file join ${::sys_libdir} extra ]
     foreach p $::sys_staticpath {
+        if { [file normalize $p] == $extradir } {
+            set p [file join @PD_PATH@ extra]
+        }
         ::deken::preferences::create_pathentry ${pathsframe} ${row} ::deken::preferences::installpath $p
         incr row
     }
@@ -888,6 +989,10 @@ proc ::deken::preferences::create {mytoplevel} {
     labelframe $mytoplevel.install -text [_ "Installation options:" ] -padx 5 -pady 5 -borderwidth 1
     pack $mytoplevel.install -side top -fill x -anchor w
 
+    checkbutton $mytoplevel.install.verify256 -text [_ "Try to verify the libraries' checksum before (re)installing them?"] \
+        -variable ::deken::preferences::verify_sha256
+    pack $mytoplevel.install.verify256 -anchor w
+
     checkbutton $mytoplevel.install.remove -text [_ "Try to remove libraries before (re)installing them?"] \
         -variable ::deken::preferences::remove_on_install
     pack $mytoplevel.install.remove -anchor w
@@ -896,6 +1001,11 @@ proc ::deken::preferences::create {mytoplevel} {
         -variable ::deken::preferences::show_readme
     pack $mytoplevel.install.readme -anchor w
 
+    checkbutton $mytoplevel.install.keeppackage -text [_ "Keep package files after installation?"] \
+        -variable ::deken::preferences::keep_package
+    pack $mytoplevel.install.keeppackage -anchor w
+
+
     checkbutton $mytoplevel.install.add_to_path -text [_ "Should newly installed libraries be added to Pd's search path?"] \
         -variable ::deken::preferences::add_to_path
     catch { $mytoplevel.install.add_to_path configure \
@@ -1005,7 +1115,9 @@ proc ::deken::preferences::apply {mytoplevel} {
     ::deken::set_install_options \
         "${::deken::preferences::remove_on_install}" \
         "${::deken::preferences::show_readme}" \
-        "${::deken::preferences::add_to_path}"
+        "${::deken::preferences::add_to_path}" \
+        "${::deken::preferences::keep_package}" \
+        "${::deken::preferences::verify_sha256}"
 }
 proc ::deken::preferences::cancel {mytoplevel} {
     ## FIXXME properly close the window/frame (for re-use in a tabbed pane)
@@ -1100,6 +1212,7 @@ proc ::deken::clicked_link {URL filename} {
         ::deken::prompt_installdir
         set installdir [ ::deken::get_writable_dir [list $::deken::installpath ] ]
     }
+    set installdir [::deken::utilities::substpath $installdir ]
     while {1} {
         if { "$installdir" == "" } {
             set msg  [_ "Please select a (writable) installation directory!"]
@@ -1164,6 +1277,20 @@ proc ::deken::clicked_link {URL filename} {
         return
     }
     ::pdwindow::debug "\n"
+    if { ! [::deken::utilities::verify_sha256 ${URL} $fullpkgfile] } {
+        if { "$::deken::verify_sha256" } {
+            if { "$::deken::keep_package" } { } {
+                catch { file delete $fullpkgfile }
+            }
+            ::pdwindow::error "\[deken\]: "
+            ::pdwindow::error [format [_ "SHA256 sum of %1\$s does not match reference %2\$s!" ] $filename ${URL}.sha256 ]
+            ::pdwindow::error "\n"
+            ::pdwindow::error [_ "aborting."]
+            ::pdwindow::error "\n"
+            return
+        }
+    }
+
     set deldir ""
     if { "$::deken::remove_on_install" } {
         if { [::deken::utilities::uninstall $installdir $extname] } {
@@ -1267,7 +1394,7 @@ proc ::deken::download_file {URL outputfilename} {
     if {[expr $ncode != 200 ]} {
         ## FIXXME: we probably should handle redirects correctly (following them...)
         set err [::http::code $httpresult]
-        ::pdwindow::debug "\[deken\] "
+        ::pdwindow::error "\[deken\] "
         ::pdwindow::error [format [_ "Unable to download from %1\$s \[%2\$s\]" ] $URL $err ]
         ::pdwindow::error "\n"
         set outputfilename ""
@@ -1444,7 +1571,7 @@ proc urldecode {str} {
 ## API draft
 
 # each backend is implemented via a single proc
-## that takes a single argument "term", the term to search fo
+## that takes a single argument "term", the term to search for
 ## an empty term indicates "search for all"
 # the backend then returns a list of results
 ## each result is a list of the following elements:
